=== modified file 'bzrlib/lazy_import.py'
--- bzrlib/lazy_import.py	2006-09-11 20:03:50 +0000
+++ bzrlib/lazy_import.py	2006-09-11 21:56:26 +0000
@@ -19,6 +19,7 @@
 This includes waiting to import a module until it is actually used.
 """
 
+import re
 import sys
 
 
@@ -146,3 +147,49 @@
                 module_path=child_path, member=None,
                 children=grandchildren)
         return module
+
+
+def _convert_import_str_to_map(import_str, imports):
+    """This converts a import string into a list of imports.
+
+    This only understands 'import foo, foo.bar, foo.bar.baz as bing'
+
+    :param import_str: The import string to process
+    :param imports: The current map of all imports
+    """
+    assert import_str.startswith('import ')
+    import_str = import_str[len('import '):]
+
+    for path in import_str.split(','):
+        path = path.strip()
+        as_hunks = path.split(' as ')
+        if len(as_hunks) == 2:
+            # We have 'as' so this is a different style of import
+            # 'import foo.bar.baz as bing' creates a local variable
+            # named 'bing' which points to 'foo.bar.baz'
+            name = as_hunks[1].strip()
+            module_path = as_hunks[0].strip().split('.')
+            assert name not in imports
+            # No children available in 'import foo as bar'
+            imports[name] = (module_path, None, {})
+        else:
+            # Now we need to handle
+            module_path = path.split('.')
+            name = module_path[0]
+            if name not in imports:
+                # This is a new import that we haven't seen before
+                module_def = ([name], None, {})
+                imports[name] = module_def
+            else:
+                module_def = imports[name]
+
+            cur_path = [name]
+            cur = module_def[2]
+            for child in module_path[1:]:
+                cur_path.append(child)
+                if child in cur:
+                    cur = cur[child]
+                else:
+                    next = (cur_path[:], None, {})
+                    cur[child] = next
+                    cur = next[2]

=== modified file 'bzrlib/tests/test_lazy_import.py'
--- bzrlib/tests/test_lazy_import.py	2006-09-11 20:53:44 +0000
+++ bzrlib/tests/test_lazy_import.py	2006-09-11 21:56:26 +0000
@@ -529,6 +529,7 @@
         # root5.sub5 should still be lazy, but not re-import root5
         self.assertEqual(InstrumentedImportReplacer,
                          object.__getattribute__(root5.sub5, '__class__'))
+
         # Accessing root5.sub5.submoda5 should import sub5, but not either
         # of the sub objects (they should be available as lazy objects
         self.assertEqual(InstrumentedImportReplacer,
@@ -568,3 +569,97 @@
                           ('_import', 'submodb5'),
                           ('import', submodb_path, []),
                          ], self.actions)
+
+
+class TestConvertImportToList(TestCase):
+    """Directly test the conversion from import strings to lists"""
+
+    def check_result(self, expected, import_strings):
+        imports = {}
+        for import_str in import_strings:
+            lazy_import._convert_import_str_to_map(import_str, imports)
+        self.assertEqual(expected, imports,
+                         'Import of %r was not converted correctly'
+                         ' %s != %s' % (import_strings, expected, imports))
+
+    def test_import_one(self):
+        self.check_result({'one':(['one'], None, {}),
+                          }, ['import one'])
+
+    def test_import_one_two(self):
+        one_two_map = {'one':(['one'], None,
+                              {'two':(['one', 'two'], None, {}),
+                              }),
+                      }
+        self.check_result(one_two_map, ['import one.two'])
+        self.check_result(one_two_map, ['import one, one.two'])
+        self.check_result(one_two_map, ['import one', 'import one.two'])
+        self.check_result(one_two_map, ['import one.two', 'import one'])
+
+    def test_import_one_two_three(self):
+        one_two_three_map = {
+            'one':(['one'], None,
+                   {'two':(['one', 'two'], None,
+                           {'three':(['one', 'two', 'three'], None, {}),
+                           }),
+                   }),
+        }
+        self.check_result(one_two_three_map, ['import one.two.three'])
+        self.check_result(one_two_three_map, ['import one, one.two.three'])
+        self.check_result(one_two_three_map, ['import one',
+                                              'import one.two.three'])
+        self.check_result(one_two_three_map, ['import one.two.three',
+                                              'import one'])
+
+    def test_import_one_as_x(self):
+        self.check_result({'x':(['one'], None, {}),
+                          }, ['import one as x'])
+
+    def test_import_one_two_as_x(self):
+        self.check_result({'x':(['one', 'two'], None, {}),
+                          }, ['import one.two as x'])
+
+    def test_import_mixed(self):
+        mixed = {'x':(['one', 'two'], None, {}),
+                 'one':(['one'], None,
+                       {'two':(['one', 'two'], None, {}),
+                       }),
+                }
+        self.check_result(mixed, ['import one.two as x, one.two'])
+        self.check_result(mixed, ['import one.two as x', 'import one.two'])
+        self.check_result(mixed, ['import one.two', 'import one.two as x'])
+
+    def test_import_with_as(self):
+        self.check_result({'fast':(['fast'], None, {})}, ['import fast'])
+
+# These will be used when we want to check converting 'import foo' into
+# the parameters necessary for creating ImportReplacer objects
+#    def check_result(self, expected, import_string):
+#        val = lazy_import._convert_import_to_list(import_string)
+#        self.assertEqual(expected, val)
+#
+#    def test_import_one(self):
+#        self.check_result(['one', ['one'], None, []], 'import one')
+#
+#    def test_import_one_two(self):
+#        self.check_result(['one', ['one'], None,
+#                            [('two', ['one', 'two'], [])]
+#                          ],
+#                          'import one.two')
+#
+#    def test_import_one_two_three(self):
+#        self.check_result(['one', ['one'], None,
+#                            [('two', ['one', 'two'],
+#                                [('three', ['one', 'two', 'three'], []),
+#                                ]
+#                             ),
+#                            ]
+#                          ],
+#                          'import one.two.three')
+#
+#    def test_from_one_import_two(self):
+#        self.check_result(['two', ['one'], 'two', []], 'from one import two')
+#
+#    def test_from_one_two_import_three(self):
+#        self.check_result(['three', ['one', 'two'], 'three', []],
+#                          'from one.two import three')

